
import {DateTime} from 'luxon';

import config from 'ui/config';

/*
* Avoids having two Fahrenheit parameters for monitoring locations that already have both a measured Celsius and
* Fahrenheit value for corresponding data parameters.
*  Below is a listing of the known codes and counterparts
* '00020': '00021' - air temperature C:F
* '00010': '00011' - water temperature C:F
* '45589': '45590' - Temperature, internal, within equipment shelter C:F
* @param {String} parameterCode - five  digit USGS identifier for data sampled at monitoring location
* @param {Array of String} - allParameterCodes
* @return {Boolean} - isParameterMatching, true if current parameter code matches a corresponding code for
* Fahrenheit in the application state, false if it does not.
 */
export const hasMeasuredFahrenheitParameter = function(parameterCode, allParameterCodes) {
    let isParameterMatching;
    switch (parameterCode) {
        case '00020':
            isParameterMatching = allParameterCodes.includes('00021');
            break;
        case '00010':
            isParameterMatching = allParameterCodes.includes('00011');
            break;
        case '45589':
            isParameterMatching = allParameterCodes.includes('45590');
            break;
        default:
            isParameterMatching = false;
    }

    return isParameterMatching;
};

/*
 * Return true if this parameter code represents one that is calculated
 * @param {String} parameterCode
 * @return {Boolean}
 */
export const isCalculatedTemperature = function(parameterCode) {
    return parameterCode ? parameterCode.slice(-1) === config.CALCULATED_TEMPERATURE_VARIABLE_CODE :  false;
};

/*
 * Converts the parameter object for a celsius parameter to one that represents calculated Fahrenheit.
 * @param {Object} parameter
 * @return {Object} converted parameter
 */
export const getConvertedTemperatureParameter = function(parameter) {
    return {
        ...parameter,
        parameterCode: `${parameter.parameterCode}${config.CALCULATED_TEMPERATURE_VARIABLE_CODE}`,
        name: parameter.name.replace('C', 'F (calculated)'),
        description: parameter.description.replace('Celsius', 'Fahrenheit (calculated)'),
        unit: parameter.unit.replace('C', 'F')
    };
};

/*
 * Determines the period of record for parameterCode using the information in the config for IV and GW period of record
 * @param {String} - parameterCode
 * @return {Object} - has properties begin_date and end_date string properties that contain the ISO 8601 date
 * string or if no period of record, then the properties will contain the null string.
 */
export const getPeriodOfRecord = function(parameterCode) {
    const measuredParameterCode = parameterCode.replace(config.CALCULATED_TEMPERATURE_VARIABLE_CODE, '');
    const ivPeriodOfRecord = config.ivPeriodOfRecord && measuredParameterCode in config.ivPeriodOfRecord ?
        config.ivPeriodOfRecord[measuredParameterCode] : null;
    const gwPeriodOfRecord = config.gwPeriodOfRecord && measuredParameterCode in config.gwPeriodOfRecord ?
        config.gwPeriodOfRecord[measuredParameterCode] : null;

    let periodOfRecord;
    if (ivPeriodOfRecord && gwPeriodOfRecord) {
        periodOfRecord = {
            begin_date: DateTime.fromISO(ivPeriodOfRecord.begin_date) < DateTime.fromISO(gwPeriodOfRecord.begin_date) ?
                ivPeriodOfRecord.begin_date : gwPeriodOfRecord.begin_date,
            end_date: DateTime.fromISO(ivPeriodOfRecord.end_date) > DateTime.fromISO(gwPeriodOfRecord.end_date) ?
                ivPeriodOfRecord.end_date : gwPeriodOfRecord.end_date
        };
    } else if (ivPeriodOfRecord) {
        periodOfRecord = ivPeriodOfRecord;
    } else if (gwPeriodOfRecord) {
        periodOfRecord = gwPeriodOfRecord;
    } else {
        periodOfRecord = {
            begin_date: '',
            end_date: ''
        };
    }
    return periodOfRecord;
};
