import {
    getFloodStageHeight, hasFloodData, getFloodGageHeightStageIndex,
    showFloodLevels, getFloodLevels, hasFloodLevels
} from './flood-data-selector';

describe('monitoring-location/selectors/flood-data-selector', () => {

    describe('getFloodStageHeight', () => {

        it('If stages is empty,null is returned', () => {
            expect(getFloodStageHeight({
                floodData: {
                    stages: []
                },
                floodState: {
                    gageHeight: 21
                }
            })).toBeNull();
        });

        it('If stages is not empty, the gageHeight returned is set to the closest stage', () => {
            let state = {
                floodData: {
                    stages: [9, 10, 11]
                },
                floodState: {
                    gageHeight: 9.4
                }
            };

            expect(getFloodStageHeight(state)).toBe(9);

            state = {
                ...state,
                floodState: {
                    gageHeight: 9.5
                }
            };

            expect(getFloodStageHeight(state)).toBe(10);
        });

        it('If gageHeight is less than any on the stages then return the lowest stage', () => {
            expect(getFloodStageHeight({
                floodData: {
                    stages: [9, 10, 11]
                },
                floodState: {
                    gageHeight: 7.5
                }
            })).toBe(9);
        });

        it('If gageHeight is greater than any on the stages then return the highest stage', () => {
            expect(getFloodStageHeight({
                floodData: {
                    stages: [9, 10, 11]
                },
                floodState: {
                    gageHeight: 14.4
                }
            })).toBe(11);
        });
    });

    describe('hasFloodData', () => {
        it('Return false if no flood stages are available', () => {
            expect(hasFloodData({
                floodData: {
                    stages: []
                }
            })).toBeFalsy();
        });

        it('return true if flood stages are available', () => {
            expect(hasFloodData({
                floodData: {
                    stages: [9, 10, 11]
                }
            })).toBeTruthy();
        });
    });

    describe('hasFloodLevels', () => {
        it('Return false if no flood levels are available', () => {
            expect(hasFloodLevels({
                floodData: {
                    floodLevels: null
                }
            })).toBeFalsy();
        });

        it('return true if flood levels are available', () => {
            expect(hasFloodLevels({
                floodData: {
                    floodLevels: {
                        site_no: '07144100',
                        action_stage: '20',
                        flood_stage: '22',
                        moderate_flood_stage: '25',
                        major_flood_stage: '26'
                    }
                }
            })).toBeTruthy();
        });
    });

    describe('getFloodLevels', () => {
        it('Expects null if no flood levels are defined', () => {
           expect(getFloodLevels({
               floodData: {
                   floodLevels: null
               }
           })).toBeNull();
        });

        it('Flood levels are returned', () => {
            expect(getFloodLevels({
                floodData: {
                    floodLevels: {
                        site_no: '07144100',
                        action_stage: '20',
                        flood_stage: '22.5',
                        moderate_flood_stage: '25',
                        major_flood_stage: '26'
                    }
                }
            })).toEqual({
                site_no: '07144100',
                action_stage: '20',
                flood_stage: '22.5',
                moderate_flood_stage: '25',
                major_flood_stage: '26'
            });
        });
    });

    describe('showFloodLevels', () => {
        it('Return false if flood levels exist but no primary parameter exists', () => {
            expect(showFloodLevels({
                floodData: {
                    floodLevels: {
                        site_no: '07144100',
                        action_stage: '20',
                        flood_stage: '22',
                        moderate_flood_stage: '25',
                        major_flood_stage: '26'
                    }
                },
                hydrographData: {}
            })).toBeFalsy();
        });

        it('Return false if flood levels should not be visible due to parameter code', () => {
            expect(showFloodLevels({
                floodData: {
                    floodLevels: {
                        site_no: '07144100',
                        action_stage: '20',
                        flood_stage: '22',
                        moderate_flood_stage: '25',
                        major_flood_stage: '26'
                    }
                },
                hydrographData: {
                    primaryIVData: {
                        parameter: {
                            parameterCode: '00060'
                        }
                    }
                }
            })).toBeFalsy();
        });

        it('Return false if flood levels should not be visible due to no flood levels', () => {
            expect(showFloodLevels({
                floodData: {
                    floodLevels: null
                },
                hydrographData: {
                    primaryIVData: {
                        parameter: {
                            parameterCode: '00065'
                        }
                    }
                }
            })).toBeFalsy();
        });

        it('Return true if flood levels should be visible', () => {
            expect(showFloodLevels({
                floodData: {
                    floodLevels: {
                        site_no: '07144100',
                        action_stage: '20',
                        flood_stage: '22',
                        moderate_flood_stage: '25',
                        major_flood_stage: '26'
                    }
                },
                hydrographData: {
                    primaryIVData: {
                        parameter: {
                            parameterCode: '00065'
                        }
                    }
                }
            })).toBeTruthy();
        });
    });

    describe('getFloodGageHeightStageIndex', () => {
        it('If stages is empty,null is returned', () => {
            expect(getFloodGageHeightStageIndex({
                floodData: {
                    stages: []
                },
                floodState: {
                    gageHeight: 21
                }
            })).toBeNull();
        });

        it('If stages is not empty, the index returned is set to the closest stage', () => {
            let state = {
                floodData: {
                    stages: [9, 10, 11]
                },
                floodState: {
                    gageHeight: 9.4
                }
            };

            expect(getFloodGageHeightStageIndex(state)).toBe(0);

            state = {
                ...state,
                floodState: {
                    gageHeight: 9.5
                }
            };

            expect(getFloodGageHeightStageIndex(state)).toBe(1);
        });

        it('If gageHeight is less than any on the stages then return the 0', () => {
            expect(getFloodGageHeightStageIndex({
                floodData: {
                    stages: [9, 10, 11]
                },
                floodState: {
                    gageHeight: 7.5
                }
            })).toBe(0);
        });

        it('If gageHeight is greater than any on the stages then return the highest stage', () => {
            expect(getFloodGageHeightStageIndex({
                floodData: {
                    stages: [9, 10, 11]
                },
                floodState: {
                    gageHeight: 14.4
                }
            })).toBe(2);
        });
    });
});
