import config from 'ui/config';

import {isCompareIVDataVisible, isMedianDataVisible, getSelectedTimeSpan, getSelectedParameterCode,
    getSelectedPrimaryIVMethodID, getSelectedSecondaryIVMethodID, getGraphCursorOffset, getGraphBrushOffset, getInputsForRetrieval
} from './hydrograph-state-selector';

describe('monitoring-location/selectors/hydrograph-state-selector', () => {
    config.locationTimeZone = 'America/Chicago';
    describe('isCompareIVDataVisible', () => {
        it('returns false if no compare iv data visible', () => {
            expect(isCompareIVDataVisible({
                hydrographState: {}
            })).toBe(false);
        });

        it('Returns true if set in the store', () => {
            expect(isCompareIVDataVisible({
                hydrographState: {
                    showCompareIVData: true
                }
            })).toBe(true);
        });
    });

    describe('isMedianDataVisible', () => {
        it('returns false if no median data visible', () => {
            expect(isMedianDataVisible({
                hydrographState: {}
            })).toBe(false);
        });

        it('Returns true if set in the store', () => {
            expect(isMedianDataVisible({
                hydrographState: {
                    showMedianData: true
                }
            })).toBe(true);
        });
    });

    describe('getSelectedTimeSpan', () => {
       it('Returns null if no selected time span', () => {
           expect(getSelectedTimeSpan({
               hydrographState: {}
           })).toBeNull();
       });

       it('Returns selected time span', () => {
           expect(getSelectedTimeSpan({
               hydrographState: {
                   selectedTimeSpan: 'P45D'
               }
           })).toEqual('P45D');
       });
    });

    describe('getSelectedPrimaryIVMethodID', () => {
        it('Returns null if no selected method id', () => {
            expect(getSelectedPrimaryIVMethodID({
                hydrographState: {}
            })).toBeNull();
        });

        it('Returns selected IV method id', () => {
            expect(getSelectedPrimaryIVMethodID({
                hydrographState: {
                    selectedIVMethodID: '90649'
                }
            })).toEqual('90649');
        });
    });

    describe('getSecondarySelectedIVMethodID', () => {
        it('Returns null if no second parameter has been selected', () => {
            expect(getSelectedSecondaryIVMethodID({
                hydrographState: {}
            })).toBeNull();
        });

        it('Returns selected secondary IV method id', () => {
            expect(getSelectedSecondaryIVMethodID({
                hydrographState: {
                    selectedSecondaryIVMethodID: '69929'
                }
            })).toEqual('69929');
        });
    });

    describe('getGraphCursorOffset', () => {
        it('Returns null if no graph cursor offset', () => {
            expect(getGraphCursorOffset({
                hydrographState: {}
            })).toBeNull();
        });

        it('Returns graph cursor offset', () => {
            expect(getGraphCursorOffset({
                hydrographState: {
                    graphCursorOffset: 362866473
                }
            })).toEqual(362866473);
        });
    });

    describe('getGraphBushOffset', () => {
        it('Returns null if no graph brush offset', () => {
            expect(getGraphBrushOffset({
                hydrographState: {}
            })).toBeNull();
        });

        it('Returns graph brush offset', () => {
            expect(getGraphBrushOffset({
                hydrographState: {
                    graphBrushOffset: {
                        start: 0,
                        end: 60945089
                    }
                }
            })).toEqual({
                start: 0,
                end: 60945089
            });
        });
    });

    describe('getSelectedParameterCode', () => {
       it('Returns null if no selected parameter code', () => {
           expect(getSelectedParameterCode({
               hydrographState: {}
           })).toBeNull();
       });

       it('Returns selected parameter code', () => {
           expect(getSelectedParameterCode({
               hydrographState: {
                   selectedParameterCode: '00060'
               }
           })).toEqual('00060');
       });
    });

    describe('getInputsForRetrieval', () => {
        config.ivPeriodOfRecord = {
            '00010': {
                begin_date: '2020-01-01',
                end_date: '2021-10-03'
            },
            '00060': {
                begin_date: '2019-01-01',
                end_date: '2021-10-01'
            },
            '72019': {
                begin_date: '2019-11-11',
                end_date: '2021-11-03'
            }
        };

        config.gwPeriodOfRecord = {
            '72019': {
                begin_date: '2020-01-01',
                end_date: '2021-11-04'
            },
            '62611': {
                begin_date: '2019-01-15',
                end_date: '2021-12-04'
            }
        };
        config.locationTimeZone = 'America/Chicago';

        beforeEach(() => {
            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date('2021-10-01T10:00:00.000+00:00'));
        });

        afterEach(() => {
            jest.useRealTimers();
        });


        it('Return expected inputs when selectedTimeSpan is for days before today', () => {
            expect(getInputsForRetrieval({
                hydrographState: {
                    showCompareIVData: false,
                    showMedianData: true,
                    selectedTimeSpan: 'P30D',
                    selectedParameterCode: '00060',
                    selectedSecondaryParameterCode: null
                }
            })).toEqual({
                parameterCode: '00060',
                period: 'P30D',
                startTime: null,
                endTime: null,
                loadCompare: false,
                secondaryParameterCode: null
            });
        });
        it('Return expects inputs when selectedTimeSpan is a date range', () => {
            expect(getInputsForRetrieval({
                hydrographState: {
                    showCompareIVData: true,
                    showMedianData: false,
                    selectedTimeSpan: {
                        start: '2021-02-01',
                        end: '2021-02-06'
                    },
                    selectedParameterCode: '00060',
                    selectedSecondaryParameterCode: null
                }
            })).toEqual({
                parameterCode: '00060',
                period: null,
                startTime: '2021-02-01T00:00:00.000-06:00',
                endTime: '2021-02-06T23:59:59.999-06:00',
                loadCompare: true,
                secondaryParameterCode: null
            });
        });

        it('Expects that if the selected time span is a duration with Y at the end that the correct startTime and endTime is returned', () => {
            expect(getInputsForRetrieval({
                hydrographState: {
                    showCompareIVData: true,
                    showMedianData: false,
                    selectedTimeSpan: 'P10Y',
                    selectedParameterCode: '00060',
                    selectedSecondaryParameterCode: null
                }
            })).toEqual({
                parameterCode: '00060',
                period: null,
                startTime: '2011-10-01T00:00:00.000-05:00',
                endTime: '2021-10-01T05:00:00.000-05:00',
                loadCompare: true,
                secondaryParameterCode: null
            });
        });

        it('Expects to return the period of record if timespan is a string but not a duration', () => {
            expect(getInputsForRetrieval({
                hydrographState: {
                    showCompareIVData: true,
                    showMedianData: false,
                    selectedTimeSpan: 'periodOfRecord',
                    selectedParameterCode: '00060',
                    selectedSecondaryParameterCode: null
                }
            })).toEqual({
                parameterCode: '00060',
                period: null,
                startTime: '2019-01-01T00:00:00.000-06:00',
                endTime: '2021-10-01T05:00:00.000-05:00',
                loadCompare: true,
                secondaryParameterCode: null
            });
        });


        it('Expects to return the selected secondary parameter code when it exists in the state', () => {
            expect(getInputsForRetrieval({
                hydrographState: {
                    showCompareIVData: true,
                    showMedianData: false,
                    selectedTimeSpan: 'periodOfRecord',
                    selectedParameterCode: '00060',
                    selectedSecondaryParameterCode: '00065'
                }
            })).toEqual({
                parameterCode: '00060',
                period: null,
                startTime: '2019-01-01T00:00:00.000-06:00',
                endTime: '2021-10-01T05:00:00.000-05:00',
                loadCompare: true,
                secondaryParameterCode: '00065'
            });
        });
    });
});