import {applyMiddleware, combineReducers, createStore} from 'redux';
import {default as thunk} from 'redux-thunk';

import {MOCK_WATERWATCH_FLOOD_LEVELS} from 'ui/mock-service-data';

import * as fimDataService from 'ui/web-services/fim-data';
import * as floodLevelsService from 'ui/web-services/flood-levels';

import {Actions, floodDataReducer, floodStateReducer} from './flood-data';

describe('monitoring-location/store/flood-data module', () => {
    /* eslint no-use-before-define: 0 */
    let store;

    beforeEach(() => {
        store = createStore(
            combineReducers({
                floodData: floodDataReducer,
                floodState: floodStateReducer
            }),
            {
                floodData: {},
                floodState: {}
            },
            applyMiddleware(thunk)
        );
    });

    describe('floodDataReducer', () => {
        describe('Actions.setFloodFeatures', () => {
            it('Updates the flood features', () => {
                store.dispatch(Actions.setFloodFeatures([1, 2, 3], {
                    xmin: -87.46671436884024,
                    ymin: 39.434393043085194,
                    xmax: -87.40838667928894,
                    ymax: 39.514453931168774
                }));
                const floodData = store.getState().floodData;

                expect(floodData.stages).toEqual([1, 2, 3]);
                expect(floodData.extent).toEqual({
                    xmin: -87.46671436884024,
                    ymin: 39.434393043085194,
                    xmax: -87.40838667928894,
                    ymax: 39.514453931168774
                });
            });
        });

        describe('Actions.retrieveFIMFloodData', () => {

            beforeEach(() => {
                fimDataService.fetchFloodFeatures =
                    jest.fn().mockReturnValue(Promise.resolve(MOCK_STAGES));
                fimDataService.fetchFloodExtent =
                    jest.fn().mockReturnValue(Promise.resolve(MOCK_EXTENT));
            });

            it('Expects a public site to populate the store', async() => {
                fimDataService.fetchFIMPublicStatus =
                    jest.fn().mockReturnValue(Promise.resolve(true));
                let promise = store.dispatch(Actions.retrieveFIMFloodData('1234567'));

                return promise.then(() => {
                    const floodData = store.getState().floodData;

                    expect(floodData.stages).toEqual([28, 29, 30]);
                    expect(floodData.extent).toEqual(MOCK_EXTENT.extent);
                });
            });

            it('Expects a site that is not public to not populate the store', () => {
                fimDataService.fetchFIMPublicStatus =
                    jest.fn().mockReturnValue(Promise.resolve(false));
                let promise = store.dispatch(Actions.retrieveFIMFloodData('1234567'));

                return promise.then(() => {
                    const floodData = store.getState().floodData;

                    expect(floodData.stages).toEqual([]);
                    expect(floodData.extent).toEqual({});
                });
            });
        });

        describe('retrieveFloodLevels', () => {
            const FLOOD_LEVELS = JSON.parse(MOCK_WATERWATCH_FLOOD_LEVELS);

            it('Expects the store to be updated on successful fetches', () => {
                floodLevelsService.fetchFloodLevels =
                    jest.fn().mockReturnValue(Promise.resolve(FLOOD_LEVELS.sites[0]));
                const promise = store.dispatch(Actions.retrieveFloodLevels('12345678'));

                return promise.then(() => {
                    const floodLevels = store.getState().floodData.floodLevels;
                    expect(floodLevels.action_stage)
                        .toEqual(FLOOD_LEVELS.sites[0].action_stage);
                    expect(floodLevels.flood_stage)
                        .toEqual(FLOOD_LEVELS.sites[0].flood_stage);
                    expect(floodLevels.moderate_flood_stage)
                        .toEqual(FLOOD_LEVELS.sites[0].moderate_flood_stage);
                    expect(floodLevels.major_flood_stage)
                        .toEqual(FLOOD_LEVELS.sites[0].major_flood_stage);
                });
            });

            it('Expects the store to contain empty flood levels if calls are unsuccessful', () => {
                floodLevelsService.fetchFloodLevels =
                    jest.fn().mockReturnValue(Promise.resolve(null));
                const promise = store.dispatch(Actions.retrieveFloodLevels('12345678'));

                return promise.then(() => {
                    expect(store.getState().floodData.floodLevels).toEqual(null);
                });
            });
        });

        describe('setFloodLevels', () => {
            const FLOOD_LEVELS =
                {
                    site_no: '07144100',
                    action_stage: '20',
                    flood_stage: '22',
                    moderate_flood_stage: '25',
                    major_flood_stage: '26'
                };

            it('expect flood levels data to be updated', () => {
                store.dispatch(
                    Actions.setFloodLevels(FLOOD_LEVELS));
                const floodLevels = store.getState().floodData.floodLevels;

                expect(floodLevels.action_stage).toEqual(FLOOD_LEVELS.action_stage);
                expect(floodLevels.flood_stage).toEqual(FLOOD_LEVELS.flood_stage);
                expect(floodLevels.moderate_flood_stage).toEqual(FLOOD_LEVELS.moderate_flood_stage);
                expect(floodLevels.major_flood_stage).toEqual(FLOOD_LEVELS.major_flood_stage);
            });
        });
    });

    describe('floodStateReducer', () => {
        describe('Actions.setGageHeight', () => {
            it('Updates the gageHeight', () => {
                store.dispatch(Actions.setGageHeight(12));

                expect(store.getState().floodState.gageHeight).toBe(12);
            });
        });
    });
});

const MOCK_STAGES =
    [{
        'attributes': {
            'USGSID': '03341500',
            'STAGE': 30
        }
    }, {
        'attributes': {
            'USGSID': '03341500',
            'STAGE': 29
        }
    }, {
        'attributes': {
            'USGSID': '03341500',
            'STAGE': 28
        }
    }];

const MOCK_EXTENT = {
    extent: {
        xmin: -87.46671436884024,
        ymin: 39.434393043085194,
        xmax: -87.40838667928894,
        ymax: 39.514453931168774,
        spatialReference: {wkid: 4326, latestWkid: 4326}
    }
};