import {DateTime} from 'luxon';

import config from 'ui/config';

import {fetchGroundwaterLevels} from 'ui/web-services/groundwater-levels';

const INITIAL_DATA = {
    all: []
};
/*
 * Synchronous Redux action - update the groundwater level data
 * @param {Array of Objects} - Each object represents the datastreams property from
 *      the groundwater level field visit service.
 * @return {Function} - Redux action
 */
const updateGroundwaterLevelData = function(datastreams) {
    return {
        type: 'UPDATE_GROUNDWATER_LEVEL_DATA',
        datastreams
    };
};

/*
 * Asynchronous Redux action - fetches the latest data for groundwater level site visits and
 * updates the data
 * @param {String} siteno
 * @param {String} agencyCd
 * @returns {Function} - which returns a Promise that resolves to an array of datastreams where the observations
 *      for a datastream are sorted by ascending time and each observation has been augmented
 *      with the epochTime.
 */
export const retrieveGroundwaterLevelData = function(siteno, agencyCd) {
    return function(dispatch) {
        return fetchGroundwaterLevels({siteno, agencyCd}).then(resp => {
            /* Add Epoch time to each datastream. This way we deal with time is in one place
             * Time strings with hour or minute accuracy are interpreted as UTC.
             * Time strings with day, month, or year are interpreted as the start of the day in the site's local time zone.
             */
            let augmentedDatastreams = [];
            if (resp.properties && resp.properties.dataStreams) {
                augmentedDatastreams = resp.properties.dataStreams.map(datastream => {
                    const augmentedObservations = datastream.observations.map(observation => {
                        let timeZoneToUse;
                        switch (observation.phenomenonTimeAccuracy) {
                            case 'minute':
                            case 'hour':
                                timeZoneToUse = 'utc';
                                break;
                            case 'day':
                            case 'month':
                            case 'year':
                                timeZoneToUse = config.locationTimeZone;
                                break;
                            default:
                                timeZoneToUse = 'utc';
                        }
                        const epochTime = DateTime.fromISO(observation.phenomenonTime, {zone: timeZoneToUse})
                            .startOf(observation.phenomenonTimeAccuracy)
                            .toMillis();
                        return {
                            ...observation,
                            epochTime: epochTime
                        };
                    });
                    //Sorts in ascending time order
                    return {
                        ...datastream,
                        observations: augmentedObservations.sort((a, b) => a.epochTime - b.epochTime)
                    };
                });
            }
            dispatch(updateGroundwaterLevelData(augmentedDatastreams));
            return augmentedDatastreams;
        });
    };
};

/*
 * Slice reducer
 */
export const groundwaterLevelDataReducer = function(data = INITIAL_DATA, action) {
    switch (action.type) {
        case 'UPDATE_GROUNDWATER_LEVEL_DATA':
            return {
                ...data,
                all: action.datastreams
            };
        default: return data;
    }
};