
import {DateTime, Duration} from 'luxon';

import config from 'ui/config';
import {convertCelsiusToFahrenheit} from 'ui/utils';

import {fetchTimeSeries} from 'ui/web-services/instantaneous-values';
import {fetchSiteStatistics} from 'ui/web-services/statistics-data';
import {fetchDataFromSensorThings} from 'ui/web-services/sensor-things';

import {isCalculatedTemperature, getConvertedTemperatureParameter} from 'ml/parameter-code-utils';

const ALLOW_COMPARE_FOR_DURATIONS = config.IV_SHORTCUT_BUTTONS.map(button => button.timeSpan);

const getParameterToFetch = function(parameterCode) {
    return isCalculatedTemperature(parameterCode) ? parameterCode.slice(0, -1) : parameterCode;
};

/*
 * Synchronous Redux action which sets the time range for the timeRangeKind
 * @param {Object} timeRange
 *      @prop {Number} start = Unix epoch in milliseconds
 *      @prop {Number} end - Unix epoch in milliseconds
 * @param {String} timeRangeKind - expected values: 'current', 'prioryear'
 * @return {Object} Redux action
 */
const setHydrographTimeRange = function(timeRange, timeRangeKind) {
    return {
        type: 'SET_HYDROGRAPH_TIME_RANGE',
        timeRange,
        timeRangeKind
    };
};

/*
 * Synchronous Redux action which clears all data
 * @return {Object} Redux action
 */
const clearHydrographData = function() {
    return {
        type: 'CLEAR_HYDROGRAPH_DATA'
    };
};

/*
 * Synchronous Redux action which clears all data except for stats data
 * @return {Object} Redux action
 */
const clearNonStatsHydrographData = function() {
    return {
        type: 'CLEAR_NON_STATS_HYDROGRAPH_DATA'
    };
};

/*
 * Synchronous Redux action which sets the IV data  for the dataKind
 * @param {Object} ivData
 *      @prop {Object} parameter
 *      @prop {Object} values - Keys are the method ids.
 * @return {Object} Redux action
 */
export const addIVHydrographData = function(dataKind, ivData) {
    return {
        type: 'ADD_IV_HYDROGRAPH_DATA',
        dataKind,
        ivData
    };
};

/*
 * Synchronous Redux action which sets the statistics data
 * @param {Object} statsData
 * @return {Object} Redux action
 */
const addStatisticsData = function(statsData) {
    return {
        type: 'ADD_STATISTICS_DATA',
        statsData
    };
};

/*
 * Synchronous Redux action which sets the operational limit data. This will be the maximum and minimum value that a
 * gage can report with accuracy for a particular observed property
 * @param {Object} thresholds
 *      @prop {Object} Can contain various threhold information such as the high and low operating limits of a location
 * @return {Object} Redux action
 */
const addHydrographThresholds =  function(thresholds) {
    return {
        type: 'ADD_THRESHOLDS',
        thresholds
    };
};

/*
 * Asynchronous Redux action to fetch the IV data for siteno, parameterCode, period or startTime/endTime
 * for the dataKind
 * @param {String} siteno
 * @param {String} dataKind - expected values: 'primary', 'secondary', or 'compare'
 * @param {String} parameterCode
 * @param {String} period - ISO 8601 duration
 * @param {String} startTime - ISO 8601 time string
 * @param {String} endTime - ISO 8601 time string
 * @return {Function} that returns a Promise
 */
const retrieveIVData = function(siteno, dataKind, {parameterCode, period, startTime, endTime}) {
    return function(dispatch) {
        const isCalculatedTemperatureCode = isCalculatedTemperature(parameterCode);

        return fetchTimeSeries({
            siteno: siteno,
            parameterCode: getParameterToFetch(parameterCode),
            period: period,
            startTime: startTime,
            endTime: endTime
        }).then(data => {
            let parameter;
            let values;
            if (data.value && data.value.timeSeries && data.value.timeSeries.length) {
                const tsData = data.value.timeSeries[0];
                // Create parameter object and adjust data if parameter code is calculated
                parameter = {
                    parameterCode: tsData.variable.variableCode[0].value,
                    name: tsData.variable.variableName,
                    description: tsData.variable.variableDescription,
                    unit: tsData.variable.unit.unitCode
                };
                if (isCalculatedTemperatureCode) {
                    parameter = getConvertedTemperatureParameter(parameter);
                }

                // The 'code' for no data value comes from the web service as a number, but the point values come
                // as strings, so convert it to a string like the point values and let JavaScript compare them.
                const noDataValue = tsData.variable.noDataValue.toString();

                /*
                 * Convert the pointValue if needed to null. If
                 * the user selected parameter is a calculated parameter code (like some cases where Celsius is
                 * converted to Fahrenheit) make that change now.
                 * @param {String} noDataValue
                 * @param {String} pointValue
                 * @returns {String}
                 */
                const getPointValue = (noDataValue, pointValue) => {
                    if (pointValue === noDataValue) {
                        return null;
                    } else if (pointValue !== null && isCalculatedTemperatureCode) {
                        return parseFloat(convertCelsiusToFahrenheit(pointValue)).toFixed(2);
                    } else {
                        return pointValue;
                    }
                };
                values = tsData.values.reduce((valuesByMethodId, value) => {
                    valuesByMethodId[value.method[0].methodID] = {
                        points: value.value.map(point => {
                            return {
                                value: getPointValue(noDataValue, point.value),
                                qualifiers: point.qualifiers,
                                dateTime: DateTime.fromISO(point.dateTime).toMillis()
                            };
                        }),
                        method: {
                            ...value.method[0],
                            methodID: value.method[0].methodID.toString()
                        }
                    };
                    return valuesByMethodId;
                }, {});
            } else {
                parameter = {
                    parameterCode: parameterCode
                };
                values = {};
            }
            dispatch(addIVHydrographData(dataKind, {
                parameter,
                values
            }));
        });
    };
};

/*
 * Asynchronous Redux action to fetch the IV data for siteno, parameterCode and startTime/endTime
 * @param {String} siteno
 * @param {String} parameterCode
 * @param {String} startTime - Epoch time
 * @param {String} endTime - EpochTime
 * @return {Function} that returns a Promise
 */
export const retrievePriorYearIVData = function(siteno, {parameterCode, startTime, endTime}) {
    return function(dispatch, getState) {
        if ('compareIVData' in getState().hydrographData) {
            return Promise.resolve();
        } else {
            const priorYearStartTime = DateTime.fromMillis(startTime).minus({days: 365}).toMillis();
            const priorYearEndTime = DateTime.fromMillis(endTime).minus({days: 365}).toMillis();
            dispatch(setHydrographTimeRange({start: priorYearStartTime, end: priorYearEndTime}, 'prioryear'));
            return dispatch(retrieveIVData(siteno, 'compare', {
                parameterCode: parameterCode,
                startTime: DateTime.fromMillis(priorYearStartTime, {zone: config.locationTimeZone}).toISO(),
                endTime: DateTime.fromMillis(priorYearEndTime, {zone: config.locationTimeZone}).toISO()
            }));
        }
    };
};

/*
 * Asynchronous Redux action to fetch the median for siteno and parameterCode
 * @param {String} siteno
 * @param {String} parameterCode
 * @return {Function} that returns a Promise
 */
export const retrieveStatistics = function(siteno, parameterCode) {
    return function(dispatch, getState) {
        if ('statisticsData' in getState().hydrographData) {
            return Promise.resolve();
        } else {
            const isCalculatedParameterCode = isCalculatedTemperature(parameterCode);
            const parameterToFetch = getParameterToFetch(parameterCode);
            return fetchSiteStatistics({siteno: siteno, statType: 'All', parameterCode: parameterToFetch})
                .then(stats => {
                    let resultStats = {};
                    if (parameterToFetch in stats) {
                        Object.keys(stats[parameterToFetch]).forEach(methodID => {
                            resultStats[methodID] = stats[parameterToFetch][methodID].map(stat => {
                                const p50Va = isCalculatedParameterCode ? convertCelsiusToFahrenheit(stat.p50_va) : parseFloat(stat.p50_va);
                                const p25Va = isCalculatedParameterCode ? convertCelsiusToFahrenheit(stat.p25_va) : parseFloat(stat.p25_va);
                                const p75Va = isCalculatedParameterCode ? convertCelsiusToFahrenheit(stat.p75_va) : parseFloat(stat.p75_va);
                                const meanVa = isCalculatedParameterCode ? convertCelsiusToFahrenheit(stat.mean_va) : parseFloat(stat.mean_va);
                                const minVa = isCalculatedParameterCode ? convertCelsiusToFahrenheit(stat.min_va) : parseFloat(stat.min_va);
                                const maxVa = isCalculatedParameterCode ? convertCelsiusToFahrenheit(stat.max_va) : parseFloat(stat.max_va);
                                return {
                                    ...stat,
                                    month_nu: parseInt(stat.month_nu),
                                    day_nu: parseInt(stat.day_nu),
                                    p50_va: p50Va,
                                    p25_va: p25Va,
                                    p75_va: p75Va,
                                    mean_va: meanVa,
                                    min_va: minVa,
                                    max_va: maxVa
                                };
                            });
                        });
                    }
                    dispatch(addStatisticsData(resultStats));
                });
        }
    };
};

/**
 * Removes the unneeded data from Sensor Things and returns only the essential information for thresholds
 * @param {Object} properties - Complex object containing threshold data along with unneeded data returned from Sensor Things
 * @return {Object} A well organised object containing threshold information
 */
const cleanThresholdData = function(properties) {
    const THRESHOLD_CODES = ['Operational limit - low-Public', 'Operational limit - high-Public'];
    if (properties.Thresholds) {
        const selectedThresholds = properties.Thresholds.filter(threshold => THRESHOLD_CODES.includes(threshold.ReferenceCode));

        return selectedThresholds.map(threshold => {
            return {
                name: threshold.Name ? threshold.Name : '',
                referenceCode: threshold.ReferenceCode ? threshold.ReferenceCode : '',
                referenceValue: threshold.Periods[0] === undefined ? '' :
                    JSON.stringify(threshold.Periods[0].ReferenceValue) ? threshold.Periods[0].ReferenceValue : ''
            };
        });
    } else {
        return [];
    }
};

/**
 * Gets data for the graph thresholds and uses only what is needed and adds the data to the application state
 * @param {String} agencySiteNumberCode -- Combines the agency code and site number into something like, USGS-01646500
 * @param {String} parameterCode -- Numerical code that uniquely identifies the observed property (parameter code)
 * @return function to dispatch threshold data to the redux store
 */
export const retrieveHydrographThresholds = function(agencySiteNumberCode, parameterCode) {
    const getMethodDescription = function(timeSeriesMetaData) {
        if (timeSeriesMetaData.properties.WebDescription) {
            return timeSeriesMetaData.properties.WebDescription;
        } else if (timeSeriesMetaData.properties.SubLocationIdentifier) {
            return timeSeriesMetaData.properties.SubLocationIdentifier;
        } else {
            return '';
        }
    };

    return function(dispatch) {
        return fetchDataFromSensorThings(agencySiteNumberCode, parameterCode).then((sensorThingsData) => {
            if (Object.keys(sensorThingsData).length !== 0) {
                const operatingLimits = sensorThingsData.value
                    .filter(timeSeriesMetaData => timeSeriesMetaData.properties && timeSeriesMetaData.properties.Thresholds)
                    .map(timeSeriesMetaData => {
                        return {
                            parameterCode: parameterCode,
                            methodDescription: getMethodDescription(timeSeriesMetaData),
                            thresholdDetails: cleanThresholdData(timeSeriesMetaData.properties)
                        };
                });

                dispatch(addHydrographThresholds({
                    operatingLimits: operatingLimits
                }));
            }
        });
    };
};

/*
 * Asynchronous Redux action which clears all data and then retrieves all of the data needed to display the hydrograph.
 * The IV and groundwater levels are automatically loaded if available. Compare and statistics data
 * are loaded if requested
 * @param {String} siteno
 * @param {String} parameterCode
 * @param {String} period - ISO 8601 duration
 * @param {String} startTime - ISO 8601 time string
 * @param {String} endTime - ISO 8601 time string
 * @param {Boolean} loadCompare
 * @param {Boolean} loadStats
 * @return {Function} that returns a promise
 */
export const retrieveHydrographData = function(siteno, agencyCode, {parameterCode, secondaryParameterCode, period, startTime, endTime, loadCompare}, loadStats = false) {
    return function(dispatch) {
        const parameterToFetch = getParameterToFetch(parameterCode);
        const hasIVData = config.ivPeriodOfRecord && parameterToFetch in config.ivPeriodOfRecord;
        if (loadStats) {
            dispatch(clearHydrographData());
        } else {
            dispatch(clearNonStatsHydrographData());
        }

        let timeRange;
        if (period && period !== 'custom') {
            const now = DateTime.local();
            timeRange = {
                start: now.minus(Duration.fromISO(period)).toMillis(),
                end: now.toMillis()
            };
        } else {
            timeRange = {
                start: DateTime.fromISO(startTime).toMillis(),
                end: DateTime.fromISO(endTime).toMillis()
            };
        }
        dispatch(setHydrographTimeRange(timeRange, 'current'));

        let fetchPromises = [];
        if (hasIVData) {
            fetchPromises.push(dispatch(retrieveHydrographThresholds(`${agencyCode}-${siteno}`, parameterCode)));
            fetchPromises.push(dispatch(retrieveIVData(
                siteno, 'primary', {parameterCode, period, startTime, endTime})));
        }

        if (hasIVData && loadCompare && ALLOW_COMPARE_FOR_DURATIONS.includes(period)) {
            fetchPromises.push(dispatch(
                retrievePriorYearIVData(siteno, {
                    parameterCode: parameterCode,
                    startTime: timeRange.start,
                    endTime: timeRange.end
                })));
        }

       if (hasIVData && secondaryParameterCode) {
           fetchPromises.push(dispatch(retrieveIVData(
               siteno, 'secondary', {parameterCode: secondaryParameterCode, period, startTime, endTime})));
       }

        if (hasIVData && loadStats) {
            fetchPromises.push(dispatch(
                retrieveStatistics(siteno, parameterCode)));
        }

        return Promise.all(fetchPromises);
    };
};

export const hydrographDataReducer = function(hydrographData = {}, action) {
    switch (action.type) {
        case 'SET_HYDROGRAPH_TIME_RANGE': {
            let newData = {};
            newData[`${action.timeRangeKind}TimeRange`] = action.timeRange;
            return Object.assign({}, hydrographData, newData);
        }
        case 'CLEAR_HYDROGRAPH_DATA': {
            return {};
        }
        case 'ADD_IV_HYDROGRAPH_DATA': {
            let newData = {};
            newData[`${action.dataKind}IVData`] = action.ivData;
            return Object.assign({}, hydrographData, newData);
        }
        case 'ADD_STATISTICS_DATA': {
            return {
                ...hydrographData,
                statisticsData: action.statsData
            };
        }
        case 'ADD_THRESHOLDS': {
            return {
                // add this to general state instead of hydrographData
                ...hydrographData,
                thresholds: action.thresholds
            };
        }
        case 'CLEAR_NON_STATS_HYDROGRAPH_DATA' : {
            return {
                statisticsData: {
                    ...hydrographData.statisticsData
                }
            };
        }
        default:
            return hydrographData;
    }
};