
import merge from 'lodash/merge';

import config from 'ui/config';

import {fetchTimeSeries} from 'ui/web-services/instantaneous-values';

import {getConvertedTemperatureParameter, hasMeasuredFahrenheitParameter} from 'ml/parameter-code-utils';
import {Actions as floodStateActions} from './flood-data';
import {retrieveGroundwaterLevelData} from './groundwater-level-field-visits';

/*
 * Synchronous Redux action - update the hydrograph variables
 * @param {Object} variables - keys are parameter codes.
 * @return {Object} - Redux action
 */
const updateHydrographParameters = function(parameters) {
    return {
        type: 'UPDATE_HYDROGRAPH_PARAMETERS',
        parameters
    };
};

/*
 * Asynchronous Redux action - fetches the latest value for all parameter codes and
 * updates the store hydrograph parameter codes.
 */
export const retrieveHydrographParameters = function(siteno, agencyCd) {
    return function(dispatch) {
        let fetchPromises = [];
        if (config.ivPeriodOfRecord) {
            const fetchIVParameters = fetchTimeSeries({siteno: siteno})
                .then(series => {
                    if (series.value && series.value.timeSeries) {
                        const allParameterCodes = series.value.timeSeries.map(ts => ts.variable.variableCode[0].value);
                        return series.value.timeSeries.reduce((varsByPCode, ts) => {
                            const parameterCode = ts.variable.variableCode[0].value;
                            varsByPCode[parameterCode] = {
                                parameterCode: parameterCode,
                                name: ts.variable.variableName,
                                description: ts.variable.variableDescription,
                                unit: ts.variable.unit.unitCode,
                                hasIVData: true,
                                latestValue: ts.values[0].value[0].value
                            };
                            // If the parameter is for gage height set the initial flood gage height
                            if (parameterCode === config.GAGE_HEIGHT_PARAMETER_CODE) {
                                dispatch(floodStateActions.setGageHeight(parseFloat(ts.values[0].value[0].value)));
                            }

                            // If it is a celsius parameterCode, add a variable for calculated Fahrenheit.
                            if (config.TEMPERATURE_PARAMETERS.celsius.includes(parameterCode) &&
                                !hasMeasuredFahrenheitParameter(parameterCode, allParameterCodes)) {
                                const fahrenheitParameter = getConvertedTemperatureParameter(varsByPCode[parameterCode]);
                                varsByPCode[fahrenheitParameter.parameterCode] = fahrenheitParameter;
                            }
                            return varsByPCode;
                        }, {});
                    } else {
                        return null;
                    }
                })
                .catch(reason => {
                    console.error(reason);
                    throw reason;
                });
            fetchPromises.push(fetchIVParameters);
        }
        if (config.gwPeriodOfRecord) {
            const fetchGWLevelParameters = dispatch(retrieveGroundwaterLevelData(siteno, agencyCd))
                .then(datastreams => {
                    let gwVars = {};
                    datastreams.forEach(datastream => {
                        gwVars[datastream.USGSParameterCode] = {
                            parameterCode: datastream.USGSParameterCode,
                            name: datastream.observedPropertyName,
                            description: datastream.observedPropertyName,
                            unit: datastream.unitOfMeasureName,
                            hasGWLevelsData: true
                        };
                    });
                    return gwVars;
                });

            fetchPromises.push(fetchGWLevelParameters);
        }
        if (fetchPromises.length) {
            return Promise.all(fetchPromises).then(([ivVars, gwVars]) => {
                const mergedVars = merge({}, gwVars, ivVars);
                dispatch(updateHydrographParameters(mergedVars));
            });
        } else {
            return Promise.resolve();
        }
    };
};

export const initializeParameters = function() {
    let ivVars = {};
    if (config.ivPeriodOfRecord) {
        const ivParameterCodes = Object.keys(config.ivPeriodOfRecord);
        ivParameterCodes.forEach(parameterCode => {
            ivVars[parameterCode] = {
                parameterCode: parameterCode,
                name: '',
                description: '',
                hasIVData: true
            };

            // If it is a celsius parameterCode, add a variable for calculated Fahrenheit.
            if (config.TEMPERATURE_PARAMETERS.celsius.includes(parameterCode) &&
                !hasMeasuredFahrenheitParameter(parameterCode, ivParameterCodes)) {
                const fahrenheitParameterCode  = `${parameterCode}${config.CALCULATED_TEMPERATURE_VARIABLE_CODE}`;
                ivVars[fahrenheitParameterCode] = {
                    parameterCode: fahrenheitParameterCode,
                    name: '',
                    description: '',
                    hasIVdata: true
                };
            }
        });
    }
    let gwVars = {};
    if (config.gwPeriodOfRecord) {
        Object.keys(config.gwPeriodOfRecord).forEach(parameterCode => {
            gwVars[parameterCode] = {
                parameterCode: parameterCode,
                name: '',
                description: '',
                hasGWLevelsData: true
            };
        });
    }
    return merge({}, gwVars, ivVars);
};

export const hydrographParametersReducer = function(hydrographParameters = {}, action) {
    switch (action.type) {
        case 'UPDATE_HYDROGRAPH_PARAMETERS': {
            return {
                ...action.parameters
            };
        }
        default:
            return hydrographParameters;
    }
};