import {applyMiddleware, combineReducers, createStore} from 'redux';
import {default as thunk} from 'redux-thunk';

import config from 'ui/config';

import {INITIAL_STATE, setCompareDataVisibility, setMedianDataVisibility, setSelectedParameterCode,
    setSelectedIVMethodID, setSelectedSecondaryIVMethodID, setSelectedTimeSpan, setGraphCursorOffset,
    setGraphBrushOffset, clearGraphBrushOffset, hydrographStateReducer
} from './hydrograph-state';

describe('monitoring-location/store/hydrograph-state', () => {
    let store;

    beforeEach(() => {
        store = createStore(
            combineReducers({
                hydrographState: hydrographStateReducer
            }),
            {
                hydrographState: INITIAL_STATE
            },
            applyMiddleware(thunk)
        );
    });

    describe('hydrographStateReducer', () => {
        describe('setCompareDataVisibility', () => {
            it('sets the compare data visibility', () => {
                store.dispatch(setCompareDataVisibility(true));
                expect(store.getState().hydrographState.showCompareIVData).toBe(true);

                store.dispatch(setCompareDataVisibility(false));
                expect(store.getState().hydrographState.showCompareIVData).toBe(false);
            });
        });

        describe('setMedianDataVisibility', () => {
            it('sets the median data visibility', () => {
                store.dispatch(setMedianDataVisibility(true));
                expect(store.getState().hydrographState.showMedianData).toBe(true);

                store.dispatch(setMedianDataVisibility(false));
                expect(store.getState().hydrographState.showMedianData).toBe(false);
            });
        });

        describe('setSelectedSecondaryIVMethodID', () => {
            it('sets the id of the secondary parameter method', () => {
                store.dispatch(setSelectedSecondaryIVMethodID ('69929'));
                expect(store.getState().hydrographState.selectedSecondaryIVMethodID).toBe('69929');
            });
        });

        describe('setSelectedParameterCode', () => {
            config.ivPeriodOfRecord = {
                '00060' : {
                    begin_date: '2019-01-01',
                    end_date: '2021-10-01'
                },
                '72019': {
                    begin_date: '2019-04-01',
                    end_date: '2021-10-01'
                },
                '00010': {
                    begin_date: '2019-04-01',
                    end_date: '2021-10-01'
                }
            };

            config.gwPeriodOfRecord = {
                '72019': {
                    begin_date: '2019-04-01',
                    end_date: '2021-10-01'
                },
                '62611': {
                    begin_date: '2019-01-01',
                    end_date: '2021-10-01'
                }
            };
            it('Sets the selected parameter code', () => {
                store.dispatch(setSelectedParameterCode('00060'));

                const state = store.getState().hydrographState;
                expect(state.selectedParameterCode).toBe('00060');
                expect(state.selectedTimeSpan).toBe('P7D');
            });

            it('changes the selected time span if the current timespan is a GW shortcut duration and the parameter code has IV Data', () => {
                store.dispatch(setSelectedTimeSpan('P10Y'));
                store.dispatch(setSelectedParameterCode('00060'));

                const state = store.getState().hydrographState;
                expect(state.selectedParameterCode).toBe('00060');
                expect(state.selectedTimeSpan).toBe('P7D');
            });

            it('changes the selected time span if the current timespan is a GW shortcut duration and the temperature parameter code has IV Data', () => {
                store.dispatch(setSelectedTimeSpan('P10Y'));
                store.dispatch(setSelectedParameterCode('00010F'));

                const state = store.getState().hydrographState;
                expect(state.selectedParameterCode).toBe('00010F');
                expect(state.selectedTimeSpan).toBe('P7D');
            });

            it('does not change the time span if it is an IV shortcut when parameter code is IV', () => {
                store.dispatch(setSelectedTimeSpan('P30D'));
                store.dispatch(setSelectedParameterCode('00060'));

                const state = store.getState().hydrographState;
                expect(state.selectedParameterCode).toBe('00060');
                expect(state.selectedTimeSpan).toBe('P30D');
            });

            it('changes the selected time span if the current timespan is a IV shortcut duration and the parameter code has GW Data', () => {
                store.dispatch(setSelectedTimeSpan('P30D'));
                store.dispatch(setSelectedParameterCode('62611'));

                const state = store.getState().hydrographState;
                expect(state.selectedParameterCode).toBe('62611');
                expect(state.selectedTimeSpan).toBe('P1Y');
            });

            it('does not change the time span if it is an GW shortcut when parameter code is GW', () => {
                store.dispatch(setSelectedTimeSpan('periodOfRecord'));
                store.dispatch(setSelectedParameterCode('62611'));

                const state = store.getState().hydrographState;
                expect(state.selectedParameterCode).toBe('62611');
                expect(state.selectedTimeSpan).toBe('periodOfRecord');
            });

        });

        describe('setSelectedIVMethodID', () => {
            it('sets the selected IV method id', () => {
                store.dispatch(setSelectedIVMethodID('1345'));
                expect(store.getState().hydrographState.selectedIVMethodID).toBe('1345');
            });
        });

        describe('setSelectedTimeSpan', () => {
            it('sets the selected timeSpan', () => {
                store.dispatch(setSelectedTimeSpan('P30D'));
                expect(store.getState().hydrographState.selectedTimeSpan).toBe('P30D');

                store.dispatch(setSelectedTimeSpan({
                    start: '2021-01-15',
                    end: '2021-03-15'
                }));
                expect(store.getState().hydrographState.selectedTimeSpan).toEqual({
                    start: '2021-01-15',
                    end: '2021-03-15'
                });
            });
        });

        describe('setGraphCursorOffset', () => {
            it('sets the graph cursor offset', () => {
                store.dispatch(setGraphCursorOffset(100200300));
                expect(store.getState().hydrographState.graphCursorOffset).toBe(100200300);
            });
        });

        describe('setGraphBrushOffset', () => {
            it('sets the graph brush offset', () => {
                store.dispatch(setGraphBrushOffset(100200, 300400));

                expect(store.getState().hydrographState.graphBrushOffset).toEqual({
                    start: 100200,
                    end: 300400
                });
            });
        });

        describe('clearGraphBrushOffset', () => {
            it('clears the iv graph brush offset', () => {
                store.dispatch(setGraphBrushOffset(100200, 300400));
                store.dispatch(clearGraphBrushOffset());

                expect(store.getState().hydrographState.graphBrushOffset).toBeNull();
            });
        });
    });
});
