import {disableFetchMocks, enableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';

import {fetchFIMPublicStatus, fetchFloodExtent, fetchFloodFeatures} from './fim-data';


describe('web-services/fim-data', () => {
    beforeEach(() => {
        enableFetchMocks();
    });

    afterEach(() => {
        disableFetchMocks();
    });

    describe('web-services/fetchFIMPublicStatus', () => {
        let restoreConsole;
        beforeEach(() => {
            fetch.resetMocks();
            restoreConsole = mockConsole();
        });

        afterEach(() => {
            restoreConsole();
        });

        it('Expects response true if valid response and response indicates the site is public', async() => {
            fetch.once(`{
                            "features" :[{
                                "attributes": {
                                    "Public": 1,
                                    "SITE_NO": "12345678"
                                }
                            }]
                        }`, {status: 200});
            const resp = await fetchFIMPublicStatus('12345678');

            expect(fetch.mock.calls).toHaveLength(1);
            expect(fetch.mock.calls[0][0]).toContain('12345678');
            expect(resp).toBeTruthy();
        });

        it('Expects response false if valid response indicates the status is not public', async() => {
            fetch.once(`{
                            "features" :[{
                                "attributes": {
                                    "Public": 0,
                                    "SITE_NO": "12345678"
                                }
                            }]
                        }`, {status: 200});
            const resp = await fetchFIMPublicStatus('12345678');

            expect(resp).toBeFalsy();
        });

        it('handles a 400 bad status', async() => {
            fetch.once('{}', {status: 400});
            const resp = await fetchFIMPublicStatus('12345678');

            expect(resp).toBeFalsy();
        });

        it('handles a bad fetch', async() => {
            fetch.mockReject(new Error('fake error message'));
            const resp = await fetchFIMPublicStatus('12345678');

            expect(resp).toBeFalsy();
        });
    });

    describe('web-services/fetchFloodFeatures', () => {
        let restoreConsole;
        beforeEach(() => {
            fetch.resetMocks();
            restoreConsole = mockConsole();
        });

        afterEach(() => {
            restoreConsole();
        });

        /* eslint no-use-before-define: off */
        it('Expects valid response to return features if in response', async() => {
            fetch.once(MOCK_FLOOD_FEATURE, {status: 200});
            const resp = await fetchFloodFeatures('12345678');

            expect(fetch.mock.calls).toHaveLength(1);
            expect(fetch.mock.calls[0][0]).toContain('12345678');
            expect(resp).toEqual(JSON.parse(MOCK_FLOOD_FEATURE).features);
        });

        it('Expects valid response with no features to return an emty array', async() => {
            fetch.once('{}', {status: 200});
            const resp = await fetchFloodFeatures('12345678');

            expect(resp).toHaveLength(0);
        });

        it('handles a 400 bad status', async() => {
            fetch.once('{}', {status: 400});
            const resp = await fetchFloodFeatures('12345678');

            expect(resp).toHaveLength(0);
        });

        it('handles a bad fetch', async() => {
            fetch.mockReject(new Error('fake error message'));
            const resp = await fetchFloodFeatures('12345678');

            expect(resp).toHaveLength(0);
        });
    });

    describe('web-services/fetchFloodExtent', () => {
        let restoreConsole;
        beforeEach(() => {
            fetch.resetMocks();
            restoreConsole = mockConsole();
        });

        afterEach(() => {
            restoreConsole();
        });
        /* eslint no-use-before-define: off */

        it('Expects a valid response to return the payload', async() => {
            fetch.once(MOCK_FLOOD_EXTENT, {status: 200});
            const resp = await fetchFloodExtent('12345678');

            expect(fetch.mock.calls).toHaveLength(1);
            expect(fetch.mock.calls[0][0]).toContain('12345678');
            expect(resp).toEqual(JSON.parse(MOCK_FLOOD_EXTENT));
        });

        it('handles a 400 bad status', async() => {
            fetch.once('{}', {status: 400});
            const resp = await fetchFloodExtent('12345678');

            expect(resp).toEqual({});
        });

        it('handles a bad fetch', async() => {
            fetch.mockReject(new Error('fake error message'));
            const resp = await fetchFloodExtent('12345678');

            expect(resp).toEqual({});
        });
    });
});

const MOCK_FLOOD_FEATURE = `
{
	"displayFieldName": "USGSID",
	"fieldAliases": {
		"USGSID": "USGSID",
		"STAGE": "STAGE"
	},
	"fields": [{
		"name": "USGSID",
		"type": "esriFieldTypeString",
		"alias": "USGSID",
		"length": 254
	}, {
		"name": "STAGE",
		"type": "esriFieldTypeDouble",
		"alias": "STAGE"
	}],
	"features": [{
		"attributes": {
			"USGSID": "03341500",
			"STAGE": 30
		}
	}, {
		"attributes": {
			"USGSID": "03341500",
			"STAGE": 29
		}
	}, {
		"attributes": {
			"USGSID": "03341500",
			"STAGE": 28
		}
	}]
}
`;

const MOCK_FLOOD_EXTENT = `
{
	"extent": {
		"xmin": -84.353211731250525,
		"ymin": 34.016663666167332,
		"xmax": -84.223456338038901,
		"ymax": 34.100999075364072,
		"spatialReference": {
			"wkid": 4326,
			"latestWkid": 4326
		}
	}
}
`;
