import config from 'ui/config';

/**
 *  Formats a URL for the purpose of downloading groundwater levels data in json format
 * @param {String} siteno - A code that uniquely identifies a monitoring location
 * @param {String} agencyCd - A code for the agency
 * @param {String} format - The format of the data returned from the API, such as 'json' or 'rdb'
 * @return {String} The URL used to contact waterservices.usgs.gov
 */
export const getGroundwaterServiceURL = function(
    {
        siteno,
        agencyCd,
        format
    }) {
    return `${config.GROUNDWATER_LEVELS_ENDPOINT}?featureId=${agencyCd}-${siteno}&type=discrete&domain=groundwater_levels&f=${format}`;
};

/**
 * Fetch the groundwater levels for site, parameterCode, and period
 * @param {String} siteno
 * @param {String} agencyCd
 * @return {Promise} resolves to Object that is retrieved with ground water levels
 */
export const fetchGroundwaterLevels = async function({siteno, agencyCd}) {
    const url = getGroundwaterServiceURL({
        siteno: siteno,
        agencyCd: agencyCd,
        format: 'json'
    });
    const request = new Request(url, {
        cache: 'no-store'
    });
    try {
        const response = await fetch(request, {
            method: 'GET'
        });
        if (response.status === 200) {
            return await response.json();
        } else {
            console.error(`Received bad status, ${response.status}  at ${url}`);
            return {};
        }
    } catch (error) {
        console.error(`Failed fetch for ${url}`);
        return {};
    }
};