"""
Initialize the Water Data for the Nation Flask application.
"""
import json
import logging
from logging.handlers import TimedRotatingFileHandler
import os
import sys

from flask import Flask

__version__ = '1.3.0dev'


def _create_log_handler(log_directory=None, log_name=__name__):
    """
    Create a handler object. The logs will be streamed
    to stdout using StreamHandler if a log directory is not specified.
    If a logfile is specified, a handler will be created so logs
    will be written to the file.
    :param str log_directory: optional path of a directory where logs can be written to
    :return: a handler
    :rtype: logging.Handler
    """
    if log_directory is not None:
        log_file = '{}.log'.format(log_name)
        log_path = os.path.join(log_directory, log_file)
        log_handler = TimedRotatingFileHandler(log_path, when='midnight', backupCount=30, delay=True)
    else:
        log_handler = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - {%(pathname)s:L%(lineno)d} - %(message)s')
    log_handler.setFormatter(formatter)
    return log_handler


app = Flask(__name__.split()[0], instance_relative_config=True)  # pylint: disable=invalid-name

# Loads configuration information from config.py and instance/config.py
app.config.from_object('config')

try:
    app.config.from_pyfile('config.py')
except FileNotFoundError:
    pass

# Read lookup files and save to the app.config
with open(os.path.join(app.config.get('DATA_DIR'),
                       app.config.get('NWIS_CODE_LOOKUP_FILENAME')), 'r') as f:
    app.config['NWIS_CODE_LOOKUP'] = json.loads(f.read())

with open(os.path.join(app.config.get('DATA_DIR'),
                       app.config.get('COUNTRY_STATE_COUNTY_LOOKUP_FILENAME')), 'r') as f:
    app.config['COUNTRY_STATE_COUNTY_LOOKUP'] = json.loads(f.read())

with open(os.path.join(app.config.get('DATA_DIR'),
                       app.config.get('HUC_LOOKUP_FILENAME')), 'r') as f:
    app.config['HUC_LOOKUP'] = json.loads(f.read())

# Load static assets manifest file, which maps source file names to the
# corresponding versioned/hashed file name.
manifest_path = app.config.get('ASSET_MANIFEST_PATH')
if manifest_path:
    with open(manifest_path, 'r') as f:
        app.config['ASSET_MANIFEST'] = json.loads(f.read())

if app.config.get('LOGGING_ENABLED'):
    handler = _create_log_handler(log_directory=app.config.get('LOGGING_DIRECTORY'))
    # Do not set logging level in the handler.
    # Otherwise, if Flask's DEBUG is set to False,
    # all logging will be disabled.
    # Instead, set the level in the logger object.
    app.logger.setLevel(app.config.get('APP_LOGGING_LEVEL'))
    app.logger.addHandler(handler)


# setup up serving of static files by whitenoise if running in a container
if os.getenv('CONTAINER_RUN'):
    from whitenoise import WhiteNoise  # nopep8
    app.wsgi_app = WhiteNoise(app.wsgi_app, root='/home/python/assets', prefix='static/')
elif app.config.get('ENABLE_WHITENOISE'):
    from whitenoise import WhiteNoise  # nopep8
    app.wsgi_app = \
        WhiteNoise(app.wsgi_app,
                   root=os.path.join(os.path.dirname(app.config.get('PROJECT_HOME')), 'assets', 'dist', ''),
                   autorefresh=True,
                   prefix=app.config.get('STATIC_ROOT'))

from . import views  # nopep8
from . import filters  # nopep8
