"""
Class and functions for calling NWIS services and working with
the returned data.

"""
from ..utils import parse_rdb

from .. import app


async def get(session, params):
    # pylint: disable=no-member
    """
    Returns a tuple containing the request status code and a list of dictionaries that represent the contents of
    RDB file
    :param session - instance aiohttp.ClientSession
    :param dict params:
    :yields
        - status_code - status code returned from the service request
        - reason - string
        - site_data - list of dictionaries
    """
    endpoint = app.config["SITE_DATA_ENDPOINT"]
    app.logger.debug(f'Requesting data from {endpoint} and {params}')
    default_params = {
        'format': 'rdb'
    }
    default_params.update(params)
    response = await session.get(endpoint, params=default_params)
    app.logger.debug(f'Received data from {endpoint} and {params} and status {response.status}')
    if response.status == 200:
        lines = await response.text()
        return 200, response.reason, list(parse_rdb(iter(lines.splitlines())))

    return response.status, response.reason, []


async def get_site_data(session, site_no, agency_cd=''):
    """
    Get the metadata for site_no, agency_cd (which may be blank) using the additional query parameters, param
    Note that more than one dictionary can be returned if agency_cd is empty.
    :param session: instance of aiohttp.ClientSession
    :param str site_no: site identifier
    :param str agency_cd: identifier for the agency that owns the site
    :yields:
        - status - status code from response
        - reason - string
        - site_metadata - list of dict representing the data returned in the rdb file
    """
    params = {
        'sites': site_no,
        'siteOutput': 'expanded'
    }
    if agency_cd:
        params['agencyCd'] = agency_cd
    return await get(session, params)


async def get_period_of_record(session, site_no, agency_cd=''):
    """
    Get the parameters measured at the site(s).
    :param session: instance of aiohttp.ClientSession
    :param str site_no: site identifier
    :param str agency_cd: identifier for the agency that owns the site:
    :returns:
        - status - status code from response
        - reason - string
        - periodOfRecord - list of dict representing the period of record for the data available at the site
    """
    params = {
        'sites': site_no,
        'seriesCatalogOutput': 'true',
        'siteStatus': 'all'
    }
    if agency_cd:
        params['agencyCd'] = agency_cd
    return await get(session, params)


async def get_huc_sites(session, huc_cd):
    """
    Get all sites within a hydrologic unit as identified by its
    hydrologic unit code (HUC).
    :param session: instance of aiohttp.ClientSession
    :param str huc_cd: hydrologic unit code
    :returns: all sites in the specified HUC
        - status - status code from response
        - reason - string
        - sites - list of dict representing the sites in huc_cd
    """
    return await get(session, {
        'huc': huc_cd
    })


async def get_county_sites(session, state_county_cd):
    """
    Get all sites within a county.
    :param session: instance of aiohttp.ClientSession
    :param str state_county_cd: FIPS ID for a statecounty
    :returns: all sites in the specified county
        - status - status code from response
        - reason - string
        - sites - list of dict representing the site in state_county_cd
     """
    return await get(session, {
        'countyCd': state_county_cd
    })
