"""
Tests for NWISWeb service calls.

"""

import aiohttp
from aioresponses import aioresponses
import pytest

from ... import app
from ...services.nwissite import get, get_site_data, get_period_of_record, get_county_sites, get_huc_sites
from ..mock_test_data import SITE_RDB, PARAMETER_RDB

ENDPOINT = app.config['SITE_DATA_ENDPOINT']


@pytest.fixture(autouse=True)
async def aiohttp_session():
    session = aiohttp.ClientSession()
    yield session
    await session.close()


def test_successful_get_no_query_parameters(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb', status=200, body=SITE_RDB)
        status_code, _, result = event_loop.run_until_complete(get(aiohttp_session, {}))

        assert status_code == 200
        assert len(result) == 1
        assert result[0]['site_no'] == '01630500'


def test_successful_get_query_parameters(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?param1=this&param2=that&format=rdb', status=200, body=SITE_RDB)
        status_code, _, _ = event_loop.run_until_complete(get(aiohttp_session, {
            'param1': 'this',
            'param2': 'that'
        }))
    assert status_code == 200


def test_500_get(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb', status=500, reason='Bad server')
        status_code, reason, _ = event_loop.run_until_complete(get(aiohttp_session, {}))
        assert status_code == 500
        assert reason == 'Bad server'


def test_non500_error_get(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb', status=404, reason='Not found')
        status_code, reason, result = event_loop.run_until_complete(get(aiohttp_session, {}))
        assert status_code == 404
        assert reason == 'Not found'
        assert result == []


def test_successful_get_site_data_with_no_agency_cd(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&sites=01630500&siteOutput=expanded', status=200, reason='OK', body=SITE_RDB)
        status_code, reason, result = event_loop.run_until_complete(get_site_data(aiohttp_session, '01630500'))
        assert status_code == 200
        assert reason == 'OK'
        assert len(result) == 1


def test_successful_get_site_with_agency_cd(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&sites=01630500&siteOutput=expanded&agencyCd=USGS',
                 status=200, reason='OK', body=SITE_RDB)
        status_code, reason, result = event_loop.run_until_complete(get_site_data(aiohttp_session, '01630500', agency_cd='USGS'))
        assert status_code == 200
        assert reason == 'OK'
        assert len(result) == 1


def test_404_get_site(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&sites=01630500&siteOutput=expanded&agencyCd=USGS',
                 status=404, reason='Not found')
        status_code, reason, result = event_loop.run_until_complete(
            get_site_data(aiohttp_session, '01630500', agency_cd='USGS'))
        assert status_code == 404
        assert reason == 'Not found'
        assert result == []


def test_successful_get_period_of_record_with_no_agency_cd(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&sites=01630500&siteStatus=all&seriesCatalogOutput=true',
                 status=200, reason='OK', body=PARAMETER_RDB)
        status_code, reason, result = event_loop.run_until_complete(
            get_period_of_record(aiohttp_session, '01630500'))
        assert status_code == 200
        assert reason == 'OK'
        assert len(result) == 8


def test_successful_get_period_of_record_with_agency_cd(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&sites=01630500&siteStatus=all&seriesCatalogOutput=true&agencyCd=USGS',
                 status=200, reason='OK', body=PARAMETER_RDB)
        status_code, reason, result = event_loop.run_until_complete(
            get_period_of_record(aiohttp_session, '01630500', agency_cd='USGS'))
        assert status_code == 200
        assert reason == 'OK'
        assert len(result) == 8


def test_404_get_period_of_record(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&sites=01630500&siteStatus=all&seriesCatalogOutput=true&agencyCd=USGS',
                 status=404, reason='Not found')
        status_code, reason, result = event_loop.run_until_complete(
            get_period_of_record(aiohttp_session, '01630500', agency_cd='USGS'))
        assert status_code == 404
        assert reason == 'Not found'
        assert result == []


def test_successful_get_huc_sites(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&huc=07010101',
                 status=200, reason='OK', body=SITE_RDB)
        status_code, reason, result = event_loop.run_until_complete(
            get_huc_sites(aiohttp_session, '07010101'))
        assert status_code == 200
        assert reason == 'OK'
        assert len(result) == 1


def test_unsuccessful_get_huc_sites(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&huc=07010101',
                 status=404, reason='Not found')
        status_code, reason, result = event_loop.run_until_complete(
            get_huc_sites(aiohttp_session, '07010101'))
        assert status_code == 404
        assert reason == 'Not found'
        assert len(result) == 0


def test_successful_get_county_sites(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&countyCd=55003',
                 status=200, reason='OK', body=SITE_RDB)
        status_code, reason, result = event_loop.run_until_complete(
            get_county_sites(aiohttp_session, '55003'))
        assert status_code == 200
        assert reason == 'OK'
        assert len(result) == 1


def test_unsuccessful_get_county_sites(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{ENDPOINT}?format=rdb&countyCd=55003',
                 status=404, reason='Not found', body=SITE_RDB)
        status_code, reason, result = event_loop.run_until_complete(
            get_county_sites(aiohttp_session, '55003'))
        assert status_code == 404
        assert reason == 'Not found'
        assert len(result) == 0
