"""
Tests for the cooperator service calls.
"""

import json

import aiohttp
from aioresponses import aioresponses

import pytest


from ... import app
from ...services.sifta import get_cooperators


MOCK_RESPONSE = """
{"Site": "06864000", "Date": "6/19/2018", "Customers":[{"Name":"Kansas Water Office","URL":"http://www.kwo.org/","IconURL":"http://water.usgs.gov/customer/icons/6737.gif"},{"Name":"USGS - Cooperative Matching Funds","URL":"http://water.usgs.gov/coop/","IconURL":"http://water.usgs.gov/customer/icons/usgsIcon.gif"}]}
"""
MOCK_CUSTOMER_LIST = json.loads(MOCK_RESPONSE)['Customers']


@pytest.fixture(autouse=True)
async def aiohttp_session():
    session = aiohttp.ClientSession()
    yield session
    await session.close()


def test_sifta_response(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{app.config["COOPERATOR_SERVICE_ENDPOINT"]}12345', status=200, body=MOCK_RESPONSE)
        has_service_error, customers = event_loop.run_until_complete(get_cooperators(aiohttp_session, '12345'))

    assert not has_service_error
    assert customers == MOCK_CUSTOMER_LIST, 'Expected response'


def test_sifta_handling_bad_status_code(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{app.config["COOPERATOR_SERVICE_ENDPOINT"]}12345', status=500)
        has_service_error, customers = event_loop.run_until_complete(get_cooperators(aiohttp_session, '12345'))

    assert not has_service_error
    assert customers == []


def test_sifta_handling_aiohttp_exceptions(aiohttp_session, event_loop):
    with aioresponses() as mock:
        mock.get(f'{app.config["COOPERATOR_SERVICE_ENDPOINT"]}12345',
                 exception=aiohttp.client_exceptions.ServerDisconnectedError())
        has_service_error, customers = event_loop.run_until_complete(get_cooperators(aiohttp_session, '12345'))

    assert has_service_error
    assert customers == []

    with aioresponses() as mock:
        mock.get(f'{app.config["COOPERATOR_SERVICE_ENDPOINT"]}12345',
                 exception=aiohttp.client_exceptions.ClientConnectorError(
                     connection_key=None, os_error=aiohttp.client_exceptions.ClientOSError))
        has_service_error, customers = event_loop.run_until_complete(get_cooperators(aiohttp_session, '12345'))

    assert has_service_error
    assert customers == []

