"""
Unit tests for the main WDFN views.
"""

import json

from unittest import TestCase, mock

from .. import app
from ..views import __version__, inject_has_feedback_link
from ..utils import parse_rdb
from .mock_test_data import SITE_RDB, PARAMETER_RDB, MOCK_NETWORKS_RESPONSE, MOCK_NETWORK_RESPONSE


def test_has_feedback_content_processor_with_valid_endpoint():
    mock_request = mock.MagicMock()
    with mock.patch('waterdata.views.request', mock_request):
        mock_request.endpoint = 'monitoring_location'
        assert inject_has_feedback_link() == {'has_feedback_link': True}

        mock_request.endpoint = 'networks'
        assert inject_has_feedback_link() == {'has_feedback_link': True}


def test_has_feedback_content_processor_with_invalid_endpoint():
    mock_request = mock.MagicMock()
    with mock.patch('waterdata.views.request', mock_request):
        mock_request.endpoint = 'questions_comments'
        assert inject_has_feedback_link() == {'has_feedback_link': False}


class TestHomeView(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    def test_version(self):
        response = self.app_client.get('/')

        self.assertEqual(response.status_code, 200)
        self.assertIn(__version__, response.data.decode('utf-8'))


class TestQuestionsCommentsView(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    def test_successful_view(self):
        fake_email = 'test%40test.gov'
        response = self.app_client.get('/questions-comments/{}/?referring_page_type=monitoring'.format(fake_email))
        self.assertEqual(200, response.status_code)


class TestFeedbackSubmitted(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    def test_successful_view(self):
        email_send_result = 'success'
        response = self.app_client.get('/feedback-submitted/{}/'.format(email_send_result))
        self.assertEqual(200, response.status_code)


class TestProvisionalDataStatementView(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    def test_successful_view(self):
        response = self.app_client.get('/provisional-data-statement/')

        self.assertEqual(response.status_code, 200)


class TestIvDataAvailabilityStatementView(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    def test_successful_view(self):
        response = self.app_client.get('/iv-data-availability-statement/')

        self.assertEqual(response.status_code, 200)


class TestMonitoringLocationView(TestCase):
    # pylint: disable=R0902

    def setUp(self):
        self.app_client = app.test_client()
        self.test_site_number = '01630500'
        self.test_url = '{0}/?site={1}'.format(app.config['SITE_DATA_ENDPOINT'], self.test_site_number)
        self.test_rdb_text = SITE_RDB
        self.test_rdb_lines = self.test_rdb_text.split('\n')
        self.headers = {'Accept': 'application/ld+json'}

    @mock.patch('waterdata.views.get_cooperators')
    @mock.patch('waterdata.views.get_period_of_record')
    @mock.patch('waterdata.views.get_site_data')
    @mock.patch('waterdata.services.sims.SiteInformationManagementService.get_sims_data')
    def test_everything_okay(self, sims_mock, site_mock, param_mock, cooperators_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        param_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(PARAMETER_RDB.split('\n')))])
        cooperators_mock.return_value = (False, [])
        sims_mock.return_value = {'alert': 'None', 'dataObservationContext': 'None', 'endangermentStatus': 'None'}

        response = self.app_client.get('/monitoring-location/{}/?agency_cd=USGS'.format(self.test_site_number))
        self.assertEqual(response.status_code, 200)
        self.assertIn('Some Random Site', response.data.decode('utf-8'))
        self.assertIn('@context', response.data.decode('utf-8'))
        # make sure no weird escaping happens when the page responds
        self.assertIn(('https://waterdata.usgs.gov/nwisweb/graph'
                       '?agency_cd=USGS&site_no=01630500&parm_cd=00060&period=100'),
                      response.data.decode('utf-8'))

        # reset iterators for json-ld tests
        json_ld_response = self.app_client.get(
            '/monitoring-location/{}/?agency_cd=USGS'.format(self.test_site_number),
            headers=self.headers
        )
        self.assertEqual(json_ld_response.status_code, 200)
        self.assertIsInstance(json.loads(json_ld_response.data), dict)

    @mock.patch('waterdata.views.get_cooperators')
    @mock.patch('waterdata.views.get_period_of_record')
    @mock.patch('waterdata.views.get_site_data')
    def test_4xx_from_water_services(self, site_mock, param_mock, cooperators_mock):
        site_mock.return_value = (400, 'Site number is invalid.', [])
        param_mock.return_value = (400, 'Site number is invalid.', [])
        cooperators_mock.return_value = (False, [])

        response = self.app_client.get('/monitoring-location/{}/'.format(self.test_site_number))
        self.assertEqual(response.status_code, 200)
        self.assertIn('Site number is invalid.', response.data.decode('utf-8'))
        self.assertNotIn('@context', response.data.decode('utf-8'))

        json_ld_response = self.app_client.get(
            '/monitoring-location/{}/'.format(self.test_site_number),
            headers=self.headers
        )
        self.assertEqual(json_ld_response.status_code, 200)
        self.assertIsNone(json.loads(json_ld_response.data))

    @mock.patch('waterdata.views.get_cooperators')
    @mock.patch('waterdata.views.get_period_of_record')
    @mock.patch('waterdata.views.get_site_data')
    def test_5xx_from_water_services(self, site_mock, param_mock, cooperators_mock):
        site_mock.return_value = (500, '', None)
        param_mock.return_value = (500, '', None)
        cooperators_mock.return_value = (False, [])

        site_mock.get(self.test_url, status_code=500)
        response = self.app_client.get('/monitoring-location/{}/'.format(self.test_site_number))
        self.assertEqual(response.status_code, 503)

        json_ld_response = self.app_client.get(
            '/monitoring-location/{}/'.format(self.test_site_number),
            headers=self.headers
        )
        self.assertEqual(json_ld_response.status_code, 503)
        self.assertIsNone(json.loads(json_ld_response.data))


class TestHydrologicalUnitView(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    @mock.patch('waterdata.views.get_huc_sites')
    def test_huc2(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/hydrological-unit/')
        assert response.status_code == 200

    @mock.patch('waterdata.views.get_huc_sites')
    def test_some_exist(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        for huc_cd in list(app.config['HUC_LOOKUP']['hucs'].keys())[:20]:
            response = self.app_client.get('/hydrological-unit/{}/'.format(huc_cd))
            assert response.status_code == 200

    @mock.patch('waterdata.views.get_huc_sites')
    def test_404s(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/hydrological-unit/1/')
        assert response.status_code == 404

    @mock.patch('waterdata.views.get_huc_sites')
    def test_locations_list(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/hydrological-unit/01010001/monitoring-locations/')
        assert response.status_code == 200
        text = response.data.decode('utf-8')
        # There are eight instances of this site in MOCK_SITE_LIST_2.
        assert text.count('01630500') == 2, 'Expected site 01630500 in output'


class TestNetworkView(TestCase):
    # pylint: disable=R0902

    def setUp(self):
        self.app_client = app.test_client()
        self.test_url = '{0}'.format(app.config['MONITORING_LOCATIONS_OBSERVATIONS_ENDPOINT'])
        self.network = 'monitoring-locations'
        self.format = 'json'

    @mock.patch('waterdata.services.ogc.MonitoringLocationNetworkService.get_networks')
    def test_networks(self, network_mock):
        network_mock.return_value = json.loads(MOCK_NETWORKS_RESPONSE)
        response = self.app_client.get('/networks/')
        # Assert network listing works
        assert response.status_code == 200

    @mock.patch('waterdata.services.ogc.MonitoringLocationNetworkService.get_networks')
    def test_one_network(self, network_mock):
        network_mock.return_value = json.loads(MOCK_NETWORK_RESPONSE)
        response = self.app_client.get('/networks/{}/'.format(self.network))
        # Assert a known network works in gui
        assert response.status_code == 200

    @mock.patch('waterdata.services.ogc.MonitoringLocationNetworkService.get_networks')
    def test_invalid_network_json_call(self, network_mock):
        network_mock.return_value = {}
        response = self.app_client.get('/networks/monitoring-locations-invalid/')
        # Assert an invalid ogc json network call 404s
        assert response.status_code == 404


class TestCountryStateCountyView(TestCase):
    def setUp(self):
        self.app_client = app.test_client()

    @mock.patch('waterdata.views.get_county_sites')
    def test_country_level_search_us(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/states/')
        assert response.status_code == 200

    @mock.patch('waterdata.views.get_county_sites')
    def test_some_counties_exist(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        for state_cd in list(app.config['COUNTRY_STATE_COUNTY_LOOKUP']['US']['state_cd'].keys())[:20]:
            response = self.app_client.get('/states/{}/'.format(state_cd))
            assert response.status_code == 200

    @mock.patch('waterdata.views.get_county_sites')
    def test_404s_incorrect_state_code(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/states/1/')
        assert response.status_code == 404

    @mock.patch('waterdata.views.get_county_sites')
    def test_404s_incorrect_county_code(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/states/01/counties/1/')
        assert response.status_code == 404

    @mock.patch('waterdata.views.get_county_sites')
    def test_locations_list(self, site_mock):
        site_mock.return_value = (200, '', [datum for datum in parse_rdb(iter(SITE_RDB.split('\n')))])
        response = self.app_client.get('/states/23/counties/003/monitoring-locations/')
        assert response.status_code == 200
        text = response.data.decode('utf-8')
        assert text.count('01630500') == 2, 'Expected site 01630500 in output'
